/***************************************************************************
 *   Copyright (C) 2004 by David Sansome                                   *
 *   david@dave-linux                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlayout.h>

#include <klocale.h>
#include <kglobal.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qdir.h>
#include <qcombobox.h>
#include <qmap.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <kurllabel.h>
#include <kapplication.h>
#include <kfontcombo.h>
#include <qspinbox.h>
#include <ksqueezedtextlabel.h>
#include <stdlib.h>

#include "kcmgtk.h"

/*typedef KGenericFactory<KcmGtk, QWidget> KcmGtkFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_gtk, KcmGtkFactory("gtk"))*/

extern "C"
{
	KCModule *create_kcmgtk( QWidget * parent, const char * name )
	{
		KGlobal::locale()->insertCatalogue( "kcmgtk" );
		return new KcmGtk( parent, "kcmgtk" );
	}
}

void GtkRcParser::parse(QString fileName)
{
	QFile file(fileName);
	file.open(IO_ReadOnly);
	QTextStream stream(&file);
	
	QRegExp includeRe("include\\s*\"([^\"]*)\"");
	QRegExp fontRe("font_name\\s*=\\s*\"([^\"]*)\"");
	
	QStringList includes;
	
	while (1)
	{
		QString line = stream.readLine();
		if (line.isNull())
			break;
		if (line.startsWith("#"))
			continue;
		
		line = line.stripWhiteSpace();
		
		if (line.startsWith("include"))
		{
			if (includeRe.search(line) == -1)
				continue;
			if (includeRe.cap(1).endsWith("/gtk-2.0/gtkrc"))
				style = includeRe.cap(1);
		}
		if (line.startsWith("font_name"))
		{
			if (fontRe.search(line) == -1)
				continue;
			// Assume there will only be one font line
			font = fontRe.cap(1);
		}
	}
	
	file.close();
	
	int lastSpacePos = font.findRev(' ');
	if (lastSpacePos != -1)
	{
		bool ok;
		fontSize = font.right(font.length() - lastSpacePos).toInt(&ok);
		if (!ok)
			fontSize = 12;
		else
			font = font.left(lastSpacePos);
	}
}

KcmGtk::KcmGtk(QWidget *parent, const char *name, const QStringList&)
    : KCModule(parent, name), myAboutData(0)
{
	QStringList gtkSearchPaths;
	gtkSearchPaths.append("/usr");
	gtkSearchPaths.append("/usr/local");
	gtkSearchPaths.append(QDir::homeDirPath() + "/.local");
	// We should probably add some other paths here too...

	for ( QStringList::Iterator it = gtkSearchPaths.begin(); it != gtkSearchPaths.end(); ++it )
	{
		QString path = (*it) + "/share/themes/";
		QDir dir(path);
		QStringList entryList = dir.entryList(QDir::Dirs, QDir::Unsorted);
		for ( QStringList::Iterator it2 = entryList.begin(); it2 != entryList.end(); ++it2 )
		{
			if ((*it2).startsWith("."))
				continue;
			if (themes.find(*it2) != themes.end())
				continue;
			if (!QFile::exists(path + (*it2) + "/gtk-2.0/gtkrc"))
				continue;
			themes.insert((*it2), path + (*it2) + "/gtk-2.0/gtkrc");
		}
	}
	
	QBoxLayout* l = new QVBoxLayout(this);
	widget = new KcmGtkWidget(this);
	l->addWidget(widget);
	connect(widget->warning2, SIGNAL(leftClickedURL(const QString&)), KApplication::kApplication(), SLOT(invokeBrowser(const QString&)));
	connect(widget->styleGroup, SIGNAL(clicked(int)), SLOT(itemChanged()));
	connect(widget->fontGroup, SIGNAL(clicked(int)), SLOT(itemChanged()));
	connect(widget->styleBox, SIGNAL(activated(int)), SLOT(itemChanged()));
	connect(widget->styleBox, SIGNAL(activated(int)), SLOT(checkStyleOther()));
	connect(widget->fontBox, SIGNAL(activated(int)), SLOT(itemChanged()));
	connect(widget->fontBox, SIGNAL(activated(int)), SLOT(checkFontOther()));
	connect(widget->fontBox, SIGNAL(activated(int)), SLOT(updatePreview()));
	connect(widget->sizeBox, SIGNAL(valueChanged(int)), SLOT(itemChanged()));
	connect(widget->sizeBox, SIGNAL(valueChanged(int)), SLOT(checkFontOther()));
	connect(widget->sizeBox, SIGNAL(valueChanged(int)), SLOT(updatePreview()));
	
	widget->styleBox->insertStringList(themes.keys());
	
	load();
};

void KcmGtk::checkStyleOther()
{
	widget->styleGroup->setButton(widget->styleGroup->id(widget->styleOther));
}

void KcmGtk::checkFontOther()
{
	widget->fontGroup->setButton(widget->fontGroup->id(widget->fontOther));
}

QString KcmGtk::env(QString key)
{
	char* value = getenv(key.latin1());
	if (value == NULL)
		return QString::null;
	else
		return QString(value);
}


KcmGtk::~KcmGtk()
{
}


void KcmGtk::load()
{
	parser.parse(QDir::homeDirPath() + "/.gtkrc-2.0");
	
	bool usingQtEngine = false;
	widget->styleBox->setCurrentText("");
	for ( QMapIterator<QString, QString> it = themes.begin(); it != themes.end(); ++it )
	{
		if (it.data() != parser.style)
			continue;
		
		if (it.key() == "Qt")
			usingQtEngine = true;
		
		widget->styleBox->setCurrentText(it.key());
	}
	
	if (usingQtEngine)
		widget->styleGroup->setButton(widget->styleGroup->id(widget->styleKde));
	else
		widget->styleGroup->setButton(widget->styleGroup->id(widget->styleOther));
	
	if (themes.find("Qt") == themes.end())
		widget->styleKde->setEnabled(false);
	else
	{
		widget->warning1->hide();
		widget->warning2->hide();
	}
	
	widget->fontBox->setCurrentFont(parser.font);
	widget->sizeBox->setValue(parser.fontSize);
	if ((QApplication::font().family() == parser.font) && (QApplication::font().pointSize() == parser.fontSize))
		widget->fontGroup->setButton(widget->fontGroup->id(widget->fontKde));
	widget->fontPreview->setEnabled(widget->fontOther->isChecked());
}


void KcmGtk::save()
{
	QFile file(QDir::homeDirPath() + "/.gtkrc-2.0");
	file.open(IO_WriteOnly);
	QTextStream stream(&file);
	
	stream << "# This file was written by KDE\n";
	stream << "# You can edit it in the KDE control center, under \"GTK styles and fonts\"\n";
	stream << "\n";
	if (widget->styleKde->isChecked())
		stream << "include \"" << themes["Qt"] << "\"\n";
	else
		stream << "include \"" << themes[widget->styleBox->currentText()] << "\"\n";
	stream << "\n";
	stream << "style \"user-font\"\n";
	stream << "{\n";
	if (widget->fontKde->isChecked())
		stream << "\tfont_name=\"" << QApplication::font().family() << " " << QString::number(QApplication::font().pointSize()) << "\"\n";
	else
		stream << "\tfont_name=\"" << widget->fontBox->currentText() << " " + QString::number(widget->sizeBox->value()) << "\"\n";
	stream << "}\n";
	stream << "widget_class \"*\" style \"user-font\"\n";
	
	file.close();
	
	emit changed(true);
}


int KcmGtk::buttons()
{
	return KCModule::Apply;
}

void KcmGtk::itemChanged()
{
	widget->fontPreview->setEnabled(widget->fontOther->isChecked());
	emit changed(true);
}

void KcmGtk::updatePreview()
{
	widget->fontPreview->setFont(QFont(widget->fontBox->currentText(), widget->sizeBox->value()));
}


QString KcmGtk::quickHelp() const
{
	return i18n("");
}


#include "kcmgtk.moc"
